@@ -6,7 +6,7 @@
  *	License as published by the Free Software Foundation version 2.1
  *	of the License.
  *
- * Copyright (c) 2008-2009 Thomas Graf <tgraf@suug.ch>
+ * Copyright (c) 2008-2010 Thomas Graf <tgraf@suug.ch>
  */
 
 #ifndef __NETLINK_CLI_LINK_H_
@@ -15,10 +15,13 @@
 #include <netlink/route/link.h>
 #include <netlink/cli/utils.h>
 
-#define nl_cli_link_alloc_cache(sk) \
-		nl_cli_alloc_cache((sk), "link", rtnl_link_alloc_cache)
-
 extern struct rtnl_link *nl_cli_link_alloc(void);
+extern struct nl_cache *nl_cli_link_alloc_cache_family(struct nl_sock *, int);
+extern struct nl_cache *nl_cli_link_alloc_cache_family_flags(struct nl_sock *, int,
+							     unsigned int);
+extern struct nl_cache *nl_cli_link_alloc_cache(struct nl_sock *);
+extern struct nl_cache *nl_cli_link_alloc_cache_flags(struct nl_sock *,
+						      unsigned int);
 
 extern void nl_cli_link_parse_family(struct rtnl_link *, char *);
 extern void nl_cli_link_parse_name(struct rtnl_link *, char *);
@@ -26,5 +29,6 @@ extern void nl_cli_link_parse_mtu(struct rtnl_link *, char *);
 extern void nl_cli_link_parse_ifindex(struct rtnl_link *, char *);
 extern void nl_cli_link_parse_txqlen(struct rtnl_link *, char *);
 extern void nl_cli_link_parse_weight(struct rtnl_link *, char *);
+extern void nl_cli_link_parse_ifalias(struct rtnl_link *, char *);
 
 #endif
