@@ -6,7 +6,7 @@
  *	License as published by the Free Software Foundation version 2.1
  *	of the License.
  *
- * Copyright (c) 2003-2008 Thomas Graf <tgraf@suug.ch>
+ * Copyright (c) 2003-2010 Thomas Graf <tgraf@suug.ch>
  */
 
 #ifndef NETLINK_RULE_H_
@@ -16,6 +16,7 @@
 #include <netlink/cache.h>
 #include <netlink/addr.h>
 #include <netlink/route/route.h>
+#include <linux/fib_rules.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -42,34 +43,32 @@ extern int rtnl_rule_delete(struct nl_sock *, struct rtnl_rule *, int);
 /* attribute modification */
 extern void		rtnl_rule_set_family(struct rtnl_rule *, int);
 extern int		rtnl_rule_get_family(struct rtnl_rule *);
-extern void		rtnl_rule_set_prio(struct rtnl_rule *, int);
-extern int		rtnl_rule_get_prio(struct rtnl_rule *);
-extern void		rtnl_rule_set_mark(struct rtnl_rule *, uint64_t);
-extern uint64_t		rtnl_rule_get_mark(struct rtnl_rule *);
-extern void		rtnl_rule_set_table(struct rtnl_rule *, int);
-extern int		rtnl_rule_get_table(struct rtnl_rule *);
-extern void		rtnl_rule_set_dsfield(struct rtnl_rule *, int);
-extern int		rtnl_rule_get_dsfield(struct rtnl_rule *);
+extern void		rtnl_rule_set_prio(struct rtnl_rule *, uint32_t);
+extern uint32_t		rtnl_rule_get_prio(struct rtnl_rule *);
+extern void		rtnl_rule_set_mark(struct rtnl_rule *, uint32_t);
+extern uint32_t		rtnl_rule_get_mark(struct rtnl_rule *);
+extern void		rtnl_rule_set_mask(struct rtnl_rule *, uint32_t);
+extern uint32_t		rtnl_rule_get_mask(struct rtnl_rule *);
+extern void		rtnl_rule_set_table(struct rtnl_rule *, uint32_t);
+extern uint32_t		rtnl_rule_get_table(struct rtnl_rule *);
+extern void		rtnl_rule_set_dsfield(struct rtnl_rule *, uint8_t);
+extern uint8_t		rtnl_rule_get_dsfield(struct rtnl_rule *);
 extern int		rtnl_rule_set_src(struct rtnl_rule *, struct nl_addr *);
 extern struct nl_addr *	rtnl_rule_get_src(struct rtnl_rule *);
 extern int		rtnl_rule_set_dst(struct rtnl_rule *, struct nl_addr *);
 extern struct nl_addr *	rtnl_rule_get_dst(struct rtnl_rule *);
-extern void		rtnl_rule_set_src_len(struct rtnl_rule *, int);
-extern int		rtnl_rule_get_src_len(struct rtnl_rule *);
-extern void		rtnl_rule_set_dst_len(struct rtnl_rule *, int);
-extern int		rtnl_rule_get_dst_len(struct rtnl_rule *);
-
-extern void		rtnl_rule_set_action(struct rtnl_rule *, int);
-extern int		rtnl_rule_get_action(struct rtnl_rule *);
-
+extern void		rtnl_rule_set_action(struct rtnl_rule *, uint8_t);
+extern uint8_t		rtnl_rule_get_action(struct rtnl_rule *);
 extern int		rtnl_rule_set_iif(struct rtnl_rule *, const char *);
 extern char *		rtnl_rule_get_iif(struct rtnl_rule *);
-
-extern void		rtnl_rule_set_classid(struct rtnl_rule *, uint32_t);
-extern uint32_t		rtnl_rule_get_classid(struct rtnl_rule *);
-
+extern int		rtnl_rule_set_oif(struct rtnl_rule *, const char *);
+extern char *		rtnl_rule_get_oif(struct rtnl_rule *);
 extern void		rtnl_rule_set_realms(struct rtnl_rule *, uint32_t);
 extern uint32_t		rtnl_rule_get_realms(struct rtnl_rule *);
+extern void		rtnl_rule_set_goto(struct rtnl_rule *, uint32_t);
+extern uint32_t		rtnl_rule_get_goto(struct rtnl_rule *);
+extern void		rtnl_rule_set_l3mdev(struct rtnl_rule *, int);
+extern int		rtnl_rule_get_l3mdev(struct rtnl_rule *);
 
 #ifdef __cplusplus
 }
