@@ -6,16 +6,28 @@
  *	License as published by the Free Software Foundation version 2.1
  *	of the License.
  *
- * Copyright (c) 2003-2006 Thomas Graf <tgraf@suug.ch>
+ * Copyright (c) 2003-2012 Thomas Graf <tgraf@suug.ch>
  */
 
 /**
- * @ingroup utils
+ * @ingroup core_types
  * @defgroup data Abstract Data
+ *
+ * Abstract data type representing a binary data blob.
+ *
+ * Related sections in the development guide:
+ * - @core_doc{_abstract_data, Abstract Data}
+ *
  * @{
+ *
+ * Header
+ * ------
+ * ~~~~{.c}
+ * #include <netlink/data.h>
+ * ~~~~
  */
 
-#include <netlink-local.h>
+#include <netlink-private/netlink.h>
 #include <netlink/netlink.h>
 #include <netlink/utils.h>
 #include <linux/socket.h>
@@ -35,7 +47,7 @@
  * 
  * @return Newly allocated data handle or NULL
  */
-struct nl_data *nl_data_alloc(void *buf, size_t size)
+struct nl_data *nl_data_alloc(const void *buf, size_t size)
 {
 	struct nl_data *data;
 
@@ -56,17 +68,31 @@ struct nl_data *nl_data_alloc(void *buf, size_t size)
 
 	return data;
 errout:
-	nl_errno(ENOMEM);
 	return NULL;
 }
 
+/**
+ * Allocate abstract data object based on netlink attribute.
+ * @arg nla		Netlink attribute of unspecific type.
+ *
+ * Allocates a new abstract data and copies the payload of the
+ * attribute to the abstract data object.
+ * 
+ * @see nla_data_alloc
+ * @return Newly allocated data handle or NULL
+ */
+struct nl_data *nl_data_alloc_attr(const struct nlattr *nla)
+{
+	return nl_data_alloc(nla_data(nla), nla_len(nla));
+}
+
 /**
  * Clone an abstract data object.
  * @arg src		Abstract data object
  *
  * @return Cloned object or NULL
  */
-struct nl_data *nl_data_clone(struct nl_data *src)
+struct nl_data *nl_data_clone(const struct nl_data *src)
 {
 	return nl_data_alloc(src->d_data, src->d_size);
 }
@@ -82,21 +108,19 @@ struct nl_data *nl_data_clone(struct nl_data *src)
  * 
  * @return 0 on success or a negative error code
  */
-int nl_data_append(struct nl_data *data, void *buf, size_t size)
+int nl_data_append(struct nl_data *data, const void *buf, size_t size)
 {
-	if (size < 0)
-		BUG();
-
 	if (size > 0) {
-		data->d_data = realloc(data->d_data, data->d_size + size);
-		if (!data->d_data)
-			return nl_errno(ENOMEM);
+		void *d_data = realloc(data->d_data, data->d_size + size);
+		if (!d_data)
+			return -NLE_NOMEM;
 
 		if (buf)
-			memcpy(data->d_data + data->d_size, buf, size);
+			memcpy(d_data + data->d_size, buf, size);
 		else
-			memset(data->d_data + data->d_size, 0, size);
+			memset(d_data + data->d_size, 0, size);
 
+		data->d_data = d_data;
 		data->d_size += size;
 	}
 
@@ -127,9 +151,11 @@ void nl_data_free(struct nl_data *data)
  * @arg data		Abstract data object.
  * @return Data buffer or NULL if empty.
  */
-void *nl_data_get(struct nl_data *data)
+void *nl_data_get(const struct nl_data *data)
 {
-	return data->d_size > 0 ? data->d_data : NULL;
+	if (data->d_size > 0)
+		return (void*)data->d_data;
+	return NULL;
 }
 
 /**
@@ -137,7 +163,7 @@ void *nl_data_get(struct nl_data *data)
  * @arg data		Abstract data object.
  * @return Size of data buffer.
  */
-size_t nl_data_get_size(struct nl_data *data)
+size_t nl_data_get_size(const struct nl_data *data)
 {
 	return data->d_size;
 }
@@ -157,10 +183,10 @@ size_t nl_data_get_size(struct nl_data *data)
  *         a is found, respectively, to be less than, to match, or
  *         be greater than b.
  */
-int nl_data_cmp(struct nl_data *a, struct nl_data *b)
+int nl_data_cmp(const struct nl_data *a, const struct nl_data *b)
 {
-	void *a_ = nl_data_get(a);
-	void *b_ = nl_data_get(b);
+	const void *a_ = nl_data_get(a);
+	const void *b_ = nl_data_get(b);
 
 	if (a_ && b_)
 		return memcmp(a_, b_, nl_data_get_size(a));
