@@ -27,6 +27,15 @@
 static struct nl_object_ops act_obj_ops;
 static struct nl_cache_ops rtnl_act_ops;
 
+struct rtnl_act * rtnl_act_next(struct rtnl_act *act)
+{
+    if (act == NULL) {
+        return NULL;
+    }
+
+    return act->a_next;
+}
+
 int rtnl_act_append(struct rtnl_act **head, struct rtnl_act *new)
 {
 	struct rtnl_act *p_act;
