@@ -30,7 +30,9 @@
 #include <netlink/utils.h>
 #include <linux/socket.h>
 #include <stdlib.h> /* exit() */
+#ifdef HAVE_STRERROR_L
 #include <locale.h>
+#endif
 
 /**
  * Global variable indicating the desired level of debugging output.
@@ -75,7 +77,7 @@ int __nl_read_num_str_file(const char *path, int (*cb)(long, const char *))
 	FILE *fd;
 	char buf[128];
 
-	fd = fopen(path, "r");
+	fd = fopen(path, "re");
 	if (fd == NULL)
 		return -nl_syserr2nlerr(errno);
 
@@ -123,9 +125,10 @@ int __nl_read_num_str_file(const char *path, int (*cb)(long, const char *))
 
 const char *nl_strerror_l(int err)
 {
+	const char *buf;
+#ifdef HAVE_STRERROR_L
 	int errno_save = errno;
 	locale_t loc = newlocale(LC_MESSAGES_MASK, "", (locale_t)0);
-	const char *buf;
 
 	if (loc == (locale_t)0) {
 		if (errno == ENOENT)
@@ -140,6 +143,9 @@ const char *nl_strerror_l(int err)
 	}
 
 	errno = errno_save;
+#else
+	buf = strerror(err);
+#endif
 	return buf;
 }
 /** @endcond */
@@ -399,7 +405,7 @@ long nl_prob2int(const char *str)
 	if (*p && strcmp(p, "%") != 0)
 		return -NLE_INVAL;
 
-	return rint(d * NL_PROB_MAX);
+	return (long) (((d * NL_PROB_MAX) + 0.5));
 }
 
 /** @} */
@@ -427,14 +433,25 @@ static double ticks_per_usec = 1.0f;
  * Supports the environment variables:
  *   PROC_NET_PSCHED  - may point to psched file in /proc
  *   PROC_ROOT        - may point to /proc fs */ 
-static void __init get_psched_settings(void)
+static void get_psched_settings(void)
 {
 	char name[FILENAME_MAX];
 	FILE *fd;
 	int got_hz = 0;
+	static volatile int initialized = 0;
+	const char *ev;
+	NL_LOCK(mutex);
 
-	if (getenv("HZ")) {
-		long hz = strtol(getenv("HZ"), NULL, 0);
+	if (initialized == 1)
+		return;
+
+	nl_lock(&mutex);
+
+	if (initialized == 1)
+		return;
+
+	if ((ev = getenv("HZ"))) {
+		long hz = strtol(ev, NULL, 0);
 
 		if (LONG_MIN != hz && LONG_MAX != hz) {
 			user_hz = hz;
@@ -447,20 +464,19 @@ static void __init get_psched_settings(void)
 
 	psched_hz = user_hz;
 
-	if (getenv("TICKS_PER_USEC")) {
-		double t = strtod(getenv("TICKS_PER_USEC"), NULL);
+	if ((ev = getenv("TICKS_PER_USEC"))) {
+		double t = strtod(ev, NULL);
 		ticks_per_usec = t;
 	}
 	else {
-		if (getenv("PROC_NET_PSCHED"))
-			snprintf(name, sizeof(name), "%s", getenv("PROC_NET_PSCHED"));
-		else if (getenv("PROC_ROOT"))
-			snprintf(name, sizeof(name), "%s/net/psched",
-				 getenv("PROC_ROOT"));
+		if ((ev = getenv("PROC_NET_PSCHED")))
+			snprintf(name, sizeof(name), "%s", ev);
+		else if ((ev = getenv("PROC_ROOT")))
+			snprintf(name, sizeof(name), "%s/net/psched", ev);
 		else
 			strncpy(name, "/proc/net/psched", sizeof(name) - 1);
 
-		if ((fd = fopen(name, "r"))) {
+		if ((fd = fopen(name, "re"))) {
 			unsigned int ns_per_usec, ns_per_tick, nom, denom;
 
 			if (fscanf(fd, "%08x %08x %08x %08x",
@@ -480,6 +496,9 @@ static void __init get_psched_settings(void)
 			fclose(fd);
 		}
 	}
+	initialized = 1;
+
+	nl_unlock(&mutex);
 }
 
 
@@ -488,6 +507,7 @@ static void __init get_psched_settings(void)
  */
 int nl_get_user_hz(void)
 {
+	get_psched_settings();
 	return user_hz;
 }
 
@@ -496,6 +516,7 @@ int nl_get_user_hz(void)
  */
 int nl_get_psched_hz(void)
 {
+	get_psched_settings();
 	return psched_hz;
 }
 
@@ -506,6 +527,7 @@ int nl_get_psched_hz(void)
  */
 uint32_t nl_us2ticks(uint32_t us)
 {
+	get_psched_settings();
 	return us * ticks_per_usec;
 }
 
@@ -517,6 +539,7 @@ uint32_t nl_us2ticks(uint32_t us)
  */
 uint32_t nl_ticks2us(uint32_t ticks)
 {
+	get_psched_settings();
 	return ticks / ticks_per_usec;
 }
 
@@ -1195,9 +1218,9 @@ int nl_has_capability (int capability)
 			NL_CAPABILITY_NL_RECVMSGS_PEEK_BY_DEFAULT),
 		_NL_SET (3,
 			NL_CAPABILITY_VERSION_3_2_29,
-			0,
-			0,
-			0,
+			NL_CAPABILITY_XFRM_SP_SEC_CTX_LEN,
+			NL_CAPABILITY_VERSION_3_3_0,
+			NL_CAPABILITY_VERSION_3_4_0,
 			0,
 			0,
 			0,
