@@ -46,6 +46,7 @@
  * ~~~~
  */
 
+#include <netlink/xfrm/selector.h>
 #include <netlink-private/netlink.h>
 
 static void sel_destroy(struct xfrmnl_sel* sel)
@@ -198,7 +199,7 @@ void xfrmnl_sel_dump(struct xfrmnl_sel* sel, struct nl_dump_params *p)
  */
 struct nl_addr* xfrmnl_sel_get_daddr (struct xfrmnl_sel* sel)
 {
-   return sel->daddr;
+	return sel->daddr;
 }
 
 int xfrmnl_sel_set_daddr (struct xfrmnl_sel* sel, struct nl_addr* addr)
@@ -214,7 +215,7 @@ int xfrmnl_sel_set_daddr (struct xfrmnl_sel* sel, struct nl_addr* addr)
 
 struct nl_addr* xfrmnl_sel_get_saddr (struct xfrmnl_sel* sel)
 {
-   return sel->saddr;
+	return sel->saddr;
 }
 
 int xfrmnl_sel_set_saddr (struct xfrmnl_sel* sel, struct nl_addr* addr)
@@ -281,7 +282,7 @@ int xfrmnl_sel_get_family(struct xfrmnl_sel *sel)
 	return sel->family;
 }
 
-int xfrmnl_sel_set_family(struct xfrmnl_sel *sel, int family)
+int xfrmnl_sel_set_family(struct xfrmnl_sel *sel, unsigned int family)
 {
 	sel->family = family;
 
