@@ -20,6 +20,8 @@
 #include <netlink/netfilter/queue.h>
 #include <netlink/netfilter/queue_msg.h>
 
+#include <linux/netlink.h>
+
 static struct nl_sock *nf_sock;
 
 static struct nfnl_queue *alloc_queue(void)
@@ -60,9 +62,8 @@ static int event_input(struct nl_msg *msg, void *arg)
 int main(int argc, char *argv[])
 {
 	struct nl_sock *rt_sock;
-	struct nl_cache *link_cache;
 	struct nfnl_queue *queue;
-	enum nfnl_queue_copy_mode copy_mode;
+	int copy_mode;
 	uint32_t copy_range;
 	int err = 1;
 	int family;
@@ -116,7 +117,7 @@ int main(int argc, char *argv[])
 
 	rt_sock = nl_cli_alloc_socket();
 	nl_cli_connect(rt_sock, NETLINK_ROUTE);
-	link_cache = nl_cli_link_alloc_cache(rt_sock);
+	nl_cli_link_alloc_cache(rt_sock);
 
 	nl_socket_set_buffer_size(nf_sock, 1024*127, 1024*127);
 
