@@ -13,6 +13,8 @@
 #include <netlink/cli/neigh.h>
 #include <netlink/cli/link.h>
 
+#include <linux/netlink.h>
+
 static void print_usage(void)
 {
 	printf(
@@ -42,13 +44,13 @@ int main(int argc, char *argv[])
 		.dp_type = NL_DUMP_LINE,
 		.dp_fd = stdout,
 	};
- 
+
 	sock = nl_cli_alloc_socket();
 	nl_cli_connect(sock, NETLINK_ROUTE);
-	link_cache = nl_cli_link_alloc_cache(sock);
+	link_cache = nl_cli_link_alloc_cache_flags(sock, NL_CACHE_AF_ITER);
 	neigh_cache = nl_cli_neigh_alloc_cache(sock);
- 	neigh = nl_cli_neigh_alloc();
- 
+	neigh = nl_cli_neigh_alloc();
+
 	for (;;) {
 		int c, optidx = 0;
 		enum {
@@ -66,7 +68,7 @@ int main(int argc, char *argv[])
 			{ "state", 1, 0, ARG_STATE },
 			{ 0, 0, 0, 0 }
 		};
-	
+
 		c = getopt_long(argc, argv, "f:hva:l:d:", long_opts, &optidx);
 		if (c == -1)
 			break;
@@ -81,7 +83,7 @@ int main(int argc, char *argv[])
 		case ARG_FAMILY: nl_cli_neigh_parse_family(neigh, optarg); break;
 		case ARG_STATE: nl_cli_neigh_parse_state(neigh, optarg); break;
 		}
- 	}
+	}
 
 	nl_cache_dump_filter(neigh_cache, &params, OBJ_CAST(neigh));
 
