@@ -11,8 +11,7 @@
 
 #include <check.h>
 
-extern Suite *make_nl_addr_suite(void);
-extern Suite *make_nl_attr_suite(void);
+#include "util.h"
 
 static Suite *main_suite(void)
 {
